LIBRARY ieee;
USE ieee.std_logic_1164.all;
USE ieee.std_logic_arith.all;
USE IEEE.std_logic_unsigned.all;

ENTITY Bjsk IS
-- Generic parameters
GENERIC (n      :INTEGER := 3); -- Number of players
PORT(
        Clk             :IN STD_LOGIC;  -- Global Clk
        Reset           :IN STD_LOGIC;  -- Global Reset
-- from Card Generator 
        Card            :IN STD_LOGIC_VECTOR (3 downto 0); -- Card Value

-- form controller
		PlayerId        :IN STD_LOGIC_VECTOR (n-1 downto 0);
		cgIssue         : IN STD_LOGIC; -- Card has been issued
	--	cgNewHand       : IN STD_LOGIC; -- New hand
		PlStand         : IN STD_LOGIC;  --- To inform that all players are standing
--from banker
		GameOver        : IN STD_LOGIC;

-- General Outputs      
        Action          :OUT STD_LOGIC_VECTOR (2*n-3 downto 0); -- Output to the Banker
	TakeAction      : OUT STD_LOGIC ; -- To the banker to start calculating
	skBust          : OUT STD_LOGIC   -- To the controller to infrm about a player bust
---Temp outputs
--	 HPTotal        : OUT STD_LOGIC_VECTOR (4 downto 0); 
--	 Card1         : OUT STD_LOGIC_VECTOR (4 downto 0)    
);

end Bjsk;


ARCHITECTURE behavior OF Bjsk IS

type matrix is array (integer range n-1 downto 0) of STD_LOGIC_VECTOR( 4 downto 0);
type matrix1 is array (integer range n-1 downto 0 ) of STD_LOGIC_VECTOR(2 downto 0);
--type matrix2 is array (integer range n-1 downto 0 ) of STD_LOGIC_VECTOR(2 downto 0);

--signal CardCount : STD_LOGIC_VECTOR (n-1 downto 0) := (others => '0');
signal cardIssued : STD_LOGIC;
signal Optotal: matrix;  -- To keep track of total card values
signal BJflagmatrix: matrix1; -- To check numbe of cards for Blackjack
--signal PlayerPrev : matrix2 := (others => "000"); 
--signal PlayerStand : STD_LOGIC_VECTOR (n-1 downto 0); 
--constant CheckStand : STD_LOGIC_VECTOR (n-1 downto 0) := (others => '1');
--signal AceMatrix   : matrix1;

--signal FlagBJ : STD_LOGIC;
signal LastPlayer : STD_LOGIC;
--signal NewHand : STD_LOGIC;
signal HP_BJ : STD_LOGIC;  -- To check if the houseplayer has a blackjack
signal InitOptotal: STD_LOGIC;  
--signal ProcInitOptotal: STD_LOGIC;

begin
-- Card Issued
-- if card is issued then check playerId and calculate the total card value
process (Reset,GameOver,PlayerId,cardIssued,Clk,Card,cgIssue,PlStand)
variable CardValue : STD_LOGIC_VECTOR ( 4 downto 0);
variable Acematrix : matrix1;
variable TempOptotal: STD_LOGIC_VECTOR ( 4 downto 0);

variable PlayerDiff : STD_LOGIC_VECTOR (4 downto 0);
variable HPDiff : STD_LOGIC_VECTOR (4 downto 0);
begin
	if ((Reset='1') or (GameOver='1')) then  --- Reset all signals and start over
		for i in n-1 downto 0 loop
			Optotal(i)<= "00000";
			BJflagmatrix(i) <= "000";
			AceMatrix(i) := "000";
		end loop;
	else


	if (Clk='1' and Clk'EVENT ) then
		if (InitOptotal = '1') then
			Optotal <= (Others => "00000");
			BJflagmatrix <= (Others=>"000");
			Acematrix := (Others => "000");
			InitOptotal <= '0';
			TakeAction <= '0';
		end if;
		if (cgIssue = '1' )  then   -- If card is issued
			for i in n-1 downto 0 loop  --To check which player has the card issued
				if PlayerId(i) = '1' then  
					CardValue := "00000";
					CardValue(3 downto 0) := Card;
				--	Card1 <= CardValue;
					if (CardValue = "00001") then  -- To check if the card is an Ace which is a special condition
						Optotal(i) <= Optotal(i) + "01011";
						BJflagmatrix(i) <= BJflagmatrix(i) + "01"; -- increment card count
						if Optotal(i) > "10101" then
							Optotal(i) <= Optotal(i) - "01011" + "00001";
						else 
							Acematrix(i) := Acematrix(i) + "001"; -- If value of 11 is taken keep track for later use
						end if;
					else
						BJflagmatrix(i) <= BJflagmatrix(i) + "01";
						TempOptotal  := Optotal(i) + CardValue;
						Optotal(i) <= Optotal(i) +  CardValue;
						if (TempOptotal > "10101") then  -- to check if the total card value is greater than 21
							Aces : for j in 7 downto 0 loop
								if (Acematrix(i) > "000")  then-- To check if the player has any Aces whose value can be subtracted
									TempOptotal := TempOptotal - "01010";
									Acematrix(i)  := Acematrix (i) - "001";	
									if TempOptotal <= "10101" then
										exit Aces;
									end if; 
								else
									exit Aces;
								end if;
							end loop Aces;
						end if; 
						if (TempOptotal > "10101") then	 --If total greater than 21 send signal to the controller that player is bust
							skBust <= '1';
						end if;
						Optotal(i) <= TempOptotal ;
					end if;
				
					if (i = n - 1)then
						LastPlayer <= '1';
					end if;
				end if;			
			end loop;
                        -- This part is commented for later in case all players stand
			--for i in n-1 downto 0 loop
			--	if (BJflagmatrix(i) > PlayerPrev(i)) then
			--		PlayerStand(i) = '1';
			--	end if;
			--end loop;
			--if PlayerStand = Checkstand then
			--	Calculate <= '1';
			--	CountPlayer <= (OTHERS=>'0');
			--else 
			--	CountPlayer <= (OTHERS=>'0');
			--end if;
			--PlayerPrev <= BJflagmatrix;
		else
			cardIssued <= '0';
			--if (InitOptotal = '1') then
			--	Optotal <= (Others => "00000");
			--	BJflagmatrix <= (Others=>"00");
			--	ProcInitOptotal<='1';
			--end if;				
		end if;

--when all players stand and controller waits for balance ready to be set high by the banker

		if (PlStand = '1' ) then-- to check if the hand is over and scores can be calculated
			if (Optotal(n-1)="10101")then  --to check if dealer has a blackjack
				if (BJflagmatrix(n-1) = "010") then
					HP_BJ<='1';
				end if;
			end if;
				
			for i in n-2 downto 0 loop
				if (Optotal(i)="10101") then  
					if (BJflagmatrix (i)="010") then  -- if the player has a blackjack
						if (HP_BJ = '1') then
							Action(2*i+1 downto 2*i) <= "01";
						else
							Action(2*i+1 downto 2*i) <= "11";
						end if;
					else
						if (HP_BJ = '1' ) then
							Action(2*i+1 downto 2*i) <= "00";
						else
							Action(2*i+1 downto 2*i) <= "10";
						end if;
					end if;
				else
					if (HP_BJ = '1') then
						Action(2*i+1 downto 2*i) <= "00";
					else
						if (Optotal(i)>"10101") then  -- if player has card value of greater than 21
							if (Optotal(n-1)>"10101") then
								Action(2*i+1 downto 2*i) <= "01";
							else
								Action(2*i+1 downto 2*i) <= "00";
							end if;
						else                      -- Take difference of cards to check who is closer to 21
							PlayerDiff := "10101" - Optotal(i);
							HPDiff     := "10101" - Optotal(n-1);
							if (HPDiff > PlayerDiff) then
								Action(2*i+1 downto 2*i) <= "10";
							else
								Action(2*i+1 downto 2*i) <= "00";
							end if;
						end if;
					end if;
				end if;
			end loop;
			InitOptotal<='1';
			TakeAction <= '1';   -- Send signal to banker
		end if;
	end if;
	end if;
end process;
end behavior;








		





